#------------------------------------------------------------------------------
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $
#------------------------------------------------------------------------------
# $Header: /project/isv-agent/ACCLib/lib/vcs/Hagrp.pm,v 1.15 2008/10/06 05:09:34 aatre Exp $
#
# Veritas Software Corporation
#
# $Author: aatre $
# $Date: 2008/10/06 05:09:34 $
# $Revision: 1.15 $
#
# Provides: package Hagrp 
#
# Functionality: a wrapper to hagrp Functionality 
#
# Authors: 011787
#------------------------------------------------------------------------------

#----------------------
# version calls
#----------------------
use Version;
Version::TellVersion("\$Header: /project/isv-agent/ACCLib/lib/vcs/Hagrp.pm,v 1.15 2008/10/06 05:09:34 aatre Exp $::ETVHeader");
#------------------------
# end of version calls
#------------------------

#----------------------------
# Declare the package name.
#----------------------------
package Hagrp;

#---------------------------------------------
# Turn on strict for refs, subs and vars.
#---------------------------------------------
use strict;

#------------------------------------
# Include all the packages required.
#------------------------------------
use LogInt;
use Resource;

#---------------------------------
# ha commands used.
#---------------------------------
#----------------------------------------------
# Support VAD/VCSOne/VCS 5.0.. Atre 30 Jan 2008
#----------------------------------------------
my $sHagrpCmd = ( $ENV { CLUSTER_HOME } ?
                  $ENV { CLUSTER_HOME } : $ENV { VCS_HOME } ) . "/bin/hagrp";

#------------------------------------------------------------------
#  options marked with '*' are available
#
#  hagrp -add <group>
#  hagrp -delete <group>
#  hagrp -link <parent group> <child group> <relationship>
#  hagrp -unlink <parent group> <child group>
#  hagrp -clear <group> [-sys <system>]
# *hagrp -online [-nopre] <group> -sys <system>
# *hagrp -offline <group> -sys <system>
# *hagrp -state [<group> -sys <system>]
#  hagrp -switch <group> -to <system>
#  hagrp -freeze <group> [-persistent]
#  hagrp -unfreeze <group> [-persistent]
#  hagrp -enable <group> [-sys <system>]
#  hagrp -disable <group> [-sys <system>]
# *hagrp -display [<group(s)>] [-attribute <attribute(s)>]
#          [-sys <system(s)>]
#  hagrp -dep [<group(s)>]
#  hagrp -resources <group>
#  hagrp -list [<conditional(s)>]
#  hagrp -value <group> <attr> [<system>]
#  hagrp -enableresources <group>
#  hagrp -disableresources <group>
#  hagrp -flush <group> -sys <system>
#  hagrp -autoenable <group> -sys <system>
#  hagrp [-help [-modify|-link|-list]]
#------------------------------------------------------------------


=pod

------------------------------------------------------------------------------
Function: Boolean Online( IN String GroupName,
                          IN String SystemName )

Description:
  Brings the service group online on specified host with pre-online trigger.

Parameters:
  GroupName  : Name of the service group.
  SystemName : Name of the host on which the service groups needs to bring online.

Return Value:
  Boolean value from the set of {0,1} where
        0 - Returns on failure.
        1 - Returns on Success.
------------------------------------------------------------------------------

=cut

sub Online {

  my ( $sGroup,
       $sSystem ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace, 
                   10352,
                   "Hagrp",
                   "Online",
                   "Subroutine %s called with: Service Group [%s], System Name [%s]",
                   $sMyName, $sGroup, $sSystem );

  #---------------------------------------------------------
  # either works or doesn't, return code reflects status
  #
  # run command - collect stderr as well as stdout
  #---------------------------------------------------------
  my $sHagrpOut   = `$sHagrpCmd -online $sGroup -sys $sSystem 2>&1`;
  my $iReturnCode = $? >> 8;

  #---------------------------------------------------------
  # check the return code if not 0 then print an error 
  # message using the returned string and return
  #---------------------------------------------------------
  if ($iReturnCode != 0) {
    chomp $sHagrpOut;
    chomp $sHagrpOut; #seems to be 2 newlines to chomp
    LogInt::Message( LogInt::LLError,
                     10353,
                     "Hagrp",
                     "Online",
                     "Failed with message:[$sHagrpOut]" );
    return 0; #error return
  }

  return 1;
}


=pod

------------------------------------------------------------------------------
Function: Boolean OnlineNoPre( IN String GroupName,
                               IN String SystemName )

Description:
  Brings the service group online on specified host without pre-online trigger.

Parameters:
  GroupName  : Name of the service group.
  SystemName : Name of the host on which the service group needs to bring online.

Return Value:
  Boolean value from the set of {0,1} where
          0 - Returns on failure.
          1 - Returns on success.
------------------------------------------------------------------------------

=cut

sub OnlineNoPre {

  my ( $sGroup, 
       $sSystem ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10354,
                   "Hagrp",
                   "OnlineNoPre",
                   "Subroutine %s called with: Service Group [%s], System Name [%s]",
                   $sMyName, $sGroup, $sSystem );

  #--------------------------------------------------------
  # either works or doesn't, return code reflects status
  #
  # run command - collect stderr as well as stdout
  #--------------------------------------------------------
  my $sHagrpOut   = `$sHagrpCmd -online -nopre $sGroup -sys $sSystem 2>&1`;
  my $iReturnCode = $? >> 8;

  #--------------------------------------------------------
  # check the return code if not 0 then print an error 
  # message using the returned string and return
  #--------------------------------------------------------
  if ($iReturnCode != 0) {
    chomp $sHagrpOut;
    chomp $sHagrpOut; #seems to be 2 newlines to chomp
    LogInt::Message( LogInt::LLError,
                     10355,
                     "Hagrp",
                     "OnlineNoPre",
                     "Failed with message:[$sHagrpOut]" );
    return 0; #error return
  }

  return 1;
}


=pod

------------------------------------------------------------------------------
Function: Boolean Offline( IN String GroupName, 
                           IN String SystemName )

Description:
  Offline the service group on specified node.

Parameters:
  GroupName  : Name of the service group.
  SystemName : Name of the system name.

Return Value:
  Boolean value from the set of {0,1} where
            0 - The service group fails to come offline.
            1 - The service group comes offline.
------------------------------------------------------------------------------

=cut

sub Offline {

  my ( $sGroup,
       $sSystem ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10356,
                   "Hagrp",
                   "Offline",
                   "Subroutine %s called with: Service Group [%s], System Name [%s]",
                   $sMyName, $sGroup, $sSystem );

  #-------------------------------------------------
  # run command - collect stderr as well as stdout
  #-------------------------------------------------
  my $sHagrpOut = `$sHagrpCmd -offline $sGroup -sys $sSystem 2>&1`;
  chomp $sHagrpOut;
  my $iReturnCode = $? >> 8;

  #--------------------------------------------------------
  # check the return code if not 0 then print an error 
  # message using the returned string and return
  #--------------------------------------------------------
  if ($iReturnCode != 0) {
    LogInt::Message( LogInt::LLError,
                     10357,
                     "Hagrp",
                     "Offline",
                     "Failed with message:[$sHagrpOut]" );
    return 0; #error return
  }

  return 1;
}


=pod

------------------------------------------------------------------------------
Function: Boolean GetStateInfoListGivenGroup( IN String GroupName,
                                              OUT ArrayOfStrings StateInfo )

Description:
  Gets the state of the service group on all nodes in the cluster.

Parameters:
  GroupName - Name of the Service group.
  StateInfo - List containing the status of the service group on each 
              node in the cluster.
                #Group       Attribute   System         Value
                Apache1326    State      gps-lin3       |OFFLINE|
                Etc will be converted into Apache1326 State gps-lin3 |OFFLINE|

Return Value:
  Boolean value from the set of {0,1} where
        0 - The command fails.
        1 - The command succeeds.
------------------------------------------------------------------------------

=cut

sub GetStateInfoListGivenGroup {

  my ( $sGroup,
       $rlsStateInfo ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10358,
                   "Hagrp",
                   "GetStateInfoListGivenGroup",
                   "Subroutine %s called with: Service Group [%s]",
                   $sMyName, $sGroup );

  #---------------------------------------------------
  # run command - collect stderr as well as stdout
  #---------------------------------------------------
  my @lsHagrpOut = `$sHagrpCmd -state $sGroup 2>&1`;
  my $iReturnCode = $? >> 8;

  #-----------------------------------------------------
  # check the return code if not 0 then print an error 
  # message using the returned string and return
  #-----------------------------------------------------
  if ($iReturnCode != 0) {
    my $sHagrpOut = undef;
    if(@lsHagrpOut > 1) {
      $sHagrpOut = join('\n', @lsHagrpOut);
    }
    else {
      $sHagrpOut = $lsHagrpOut[0];
    }
    chomp $sHagrpOut;
    LogInt::Message( LogInt::LLError, 
                     10359,
                     "Hagrp",
                     "GetStateInfoListGivenGroup",
                     "Failed with message:[$sHagrpOut]" );
    return 0; #error return
  }

  #-----------------------------------------------------------
  # assign the output to the passed in referance
  # format is #Resource   Attribute        System    Value
  #-----------------------------------------------------------
  for my $sEachHagrpOut (@lsHagrpOut) {
    if($sEachHagrpOut =~ /^#/) {
      #ignore lines starting with '#'
    } 
    else {
      chomp $sEachHagrpOut;
      $sEachHagrpOut =~ s/\s+/ /g; #remove multiple spaces
      LogInt::Message( LogInt::LLTraceTrace,
                       10360,
                       "Hagrp",
                       "GetStateInfoListGivenGroup",
                       "Adding [$sEachHagrpOut] to rlsStateInfo" );
      push(@$rlsStateInfo, $sEachHagrpOut);
    }
  }

  my $iEntryCount = @$rlsStateInfo;
  LogInt::Message( LogInt::LLTrace,
                   10361,
                   "Hagrp",
                   "GetStateInfoListGivenGroup",
                   "Found [$iEntryCount] entries");

  return 1;
}


=pod

------------------------------------------------------------------------------
Function: Boolean GetStateGivenGroupAndSystem( IN String GroupName,
                                               IN String SystemName,
                                               OUT String State )

Description:
  Gets the status of the given service group on specified system.

Parameters:
  GroupName  : Name of the service group.
  SystemName : Name of the system on which the service group 
               status needs to be checked.
  State      : State of the service group on the system.

Return Value:
  Boolean value from the set of {0,1} where
        0 - The command fails to execute.
        1 - The command executes successfully.
------------------------------------------------------------------------------

=cut

sub GetStateGivenGroupAndSystem {

  my ( $sGroup,
       $sSystem, 
       $rsState ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10362,
                   "Hagrp",
                   "GetStateGivenGroupAndSystem",
                   "Subroutine %s called with: Service Group [%s], System Name [%s]",
                   $sMyName, $sGroup, $sSystem );

  #-------------------------------------------------
  #run command - collect stderr as well as stdout
  #-------------------------------------------------
  my $sHagrpOut = `$sHagrpCmd -state $sGroup -sys $sSystem 2>&1`;
  chomp $sHagrpOut;
  my $iReturnCode = $? >> 8;

  #-----------------------------------------------------
  # check the return code if not 0 then print an error 
  # message using the returned string and return
  #-----------------------------------------------------
  if ($iReturnCode != 0) {
    LogInt::Message( LogInt::LLError,
                     10363,
		     "Hagrp",
		     "GetStateGivenGroupAndSystem",
                     "Failed with message:[$sHagrpOut]" );
    return 0; #error return
  }

  LogInt::Message( LogInt::LLTrace,
                   10364,
		   "Hagrp",
		   "GetStateGivenGroupAndSystem",
                   "Called with Service Group:[$sGroup], system name:[$sSystem] returning [$sHagrpOut]" );

  #-----------------------------------------------
  # assign the output to the passed in referance
  #-----------------------------------------------
  $$rsState = $sHagrpOut;

  return 1;
}


=pod

------------------------------------------------------------------------------
Function: Boolean Display( IN String Groups,
                           IN String Attributes,
                           IN String Systems, 
                           OUT ArrayOfStrings Results )

Description:
  Gets all the attributes on specifies nodes for the given service groups.

Parameters:
  Groups     : List of the service groups.
  Attributes : List of attributes to be displayed.
  Systems    : List of systems/node in the cluster.
  Results    : An array containing all the groups, attributes, systems and values.
                The format of each entry will be as follows:
                #Group        Attribute      System    Value
                 Apache1326    Version      gps-lin3    1.3

Return Value:
  Boolean value from the set of {0,1} where
         0 - The command fails.
         1 - The command succeeds.
------------------------------------------------------------------------------

=cut

sub Display {

  my ( $sGroups,
       $sAttributes,
       $sSystems,
       $rlsResults ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10365,
		   "Hagrp",
		   "Display",
                   "Subroutine %s called with: Groups [%s], Attributes [%s], Systems [%s]",
                   $sMyName, $sGroups, $sAttributes, $sSystems );

  #--------------------------------------------------------------------
  # some, all, or none of the above may be populated.
  # results of the command are passed back in the $rlsResults param
  # hagrp -display [<group(s)>] [-attribute <attribute(s)>] [-sys <system(s)>]
  #--------------------------------------------------------------------
  #----------------------------------------------------------
  # go through the passed in params building up the arg list 
  # for hagrp call
  #----------------------------------------------------------

  #-------------------------------------
  # start with the generic display call
  #-------------------------------------
  my $sHagrpDisplayCmd = "$sHagrpCmd -display";

  if($sGroups)     { $sHagrpDisplayCmd .= " $sGroups"; }
  if($sAttributes) { $sHagrpDisplayCmd .= " -attribute $sAttributes"; }
  if($sSystems)    { $sHagrpDisplayCmd .= " -sys $sSystems"; }

  LogInt::Message( LogInt::LLTrace,
                   10366,
		   "Hagrp",
		   "Display",
                   "going to call $sHagrpDisplayCmd");

  #---------------------
  # call the command.
  #---------------------
  my @lsHagrpOut = `$sHagrpDisplayCmd 2>&1`;
  my $iReturnCode = $? >> 8;

  #-------------------------------------------------------
  # check the return code if not 0 then print an error 
  # message using the returned data
  #-------------------------------------------------------
  if ($iReturnCode != 0) {
    my $sHagrpOut = undef;
    if(@lsHagrpOut > 1) {
      $sHagrpOut = join('\n', @lsHagrpOut);
    }
    else {
      $sHagrpOut = $lsHagrpOut[0];
    }
    chomp $sHagrpOut;
    LogInt::Message( LogInt::LLError,
                     10367,
                     "Hagrp",
		     "Display",
                     "Failed with message:[$sHagrpOut]" );

    return 0; #error return
  }

  #-------------------------------------------------
  # assign the output to the passed in referance
  #
  # #Group    Attribute           System    Value
  # fsapvol1  AutoFailOver        global    1
  # etc..
  #-------------------------------------------------

  for my $sEachHagrpOut (@lsHagrpOut) {
    if($sEachHagrpOut =~ /^#/) {
      #ignore lines starting with '#'
    }
    else {
      chomp $sEachHagrpOut;
      $sEachHagrpOut =~ s/\s+/ /g; #remove multiple spaces
      LogInt::Message( LogInt::LLTraceTrace,
                       10368,
                       "Hagrp",
		       "Display",
                       "Adding [$sEachHagrpOut] to rlsResults" );

      push(@$rlsResults, $sEachHagrpOut);
    }
  }

  my $iEntryCount = @$rlsResults;
  LogInt::Message( LogInt::LLTrace, 
                   10369,
                   "Hagrp",
		   "Display",
                   "Found [$iEntryCount] entries" );

  return 1;
}


1; #so perl knows its included ok

